#!/usr/bin/env python

"""
dynagui.py
Copyright (C) 2006,2007  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

__version__ = "0.4.0"
__author__ = "Yannick Le Teigner"

import os
import sys
import pygtk; pygtk.require('2.0')
import gtk
import gobject
from dynagui_libs.canvas import MyCanvas
from dynagui_libs import lab_io
from dynagui_libs.lab import lab
from dynagui_libs.gui_router import GuiRouter
from dynagui_libs.dynamips_properties_dialog import DynamipsPropertiesDialog
from dynagui_libs.C3600_properties_dialog import C3600PropertiesDialog
from dynagui_libs.C7200_properties_dialog import C7200PropertiesDialog
from dynagui_libs.frsw_properties_dialog import FRSwitchPropertiesDialog
from dynagui_libs.atmsw_properties_dialog import ATMSwitchPropertiesDialog
from dynagui_libs.ethsw_properties_dialog import EthSwitchPropertiesDialog
from dynagui_libs.util import RouterSelectionDialog


ui_popup = \
'''<ui>
  <popup action='EditMenu'>
    <menuitem action='Add7200'/>
    <menuitem action='Add3600'/>
    <menuitem action='AddATMSwitch'/>
    <menuitem action='AddFRSwitch'/>
    <menuitem action='AddEthSwitch'/>
  </popup>
</ui>'''

ui_info = \
'''<ui>
  <menubar name='MenuBar'>
    <menu action='FileMenu'>
      <menuitem action='New'/>
      <menuitem action='Open'/>
      <menuitem action='Save'/>
      <menuitem action='SaveAs'/>
      <separator/>
      <menuitem action='Quit'/>
    </menu>
    <menu action='EditMenu'>
      <menuitem action='Add7200'/>
      <menuitem action='Add3600'/>
      <menuitem action='AddATMSwitch'/>
      <menuitem action='AddFRSwitch'/>
      <menuitem action='AddEthSwitch'/>
    </menu>
    <menu action='DevicesMenu'>
      <menuitem action='StartAll'/>
      <menuitem action='StopAll'/>
      <menuitem action='SuspendAll'/>
      <menuitem action='ResumeAll'/>
      <separator/>
    </menu>
    <menu action='ToolsMenu'>
      <menuitem action='OutputConfig'/>
    </menu>
    <menu action='HelpMenu'>
      <menuitem action='About'/>
    </menu>
  </menubar>
  <toolbar  name='ToolBar'>
    <toolitem action='Open'/>
    <toolitem action='Quit'/>
    <separator/>
  </toolbar>
</ui>'''


class DynaguiFrame(gtk.Window):

    def __init__(self, parent=None):
        #register_stock_icons()

        gtk.Window.__init__(self)
        try:
            self.set_screen(parent.get_screen())
        except:
            self.connect("delete_event", self.__quit)

        self.set_title("Dynagui")
        self.set_default_size(500, 400)

        self.canvas = MyCanvas(self)

        self.uimanager = gtk.UIManager()
        self.set_data("ui-manager", self.uimanager)
        self.uimanager.insert_action_group(self.__create_action_group(), 0)
        self.add_action_group = self.__create_add_action_group()
        self.uimanager.insert_action_group(self.add_action_group, 0)
        self.add_accel_group(self.uimanager.get_accel_group())

        try:
            uimanagerid = self.uimanager.add_ui_from_string(ui_info)
        except gobject.GError, msg:
            print "building menus failed: %s" % msg


        table = gtk.Table(1, 4, False)
        self.add(table)

        mbar = self.uimanager.get_widget("/MenuBar")
        table.attach(mbar,
            # X direction #          # Y direction
            0, 1,                      0, 1,
            gtk.EXPAND | gtk.FILL,     0,
            0,                         0);

        tbar = self.uimanager.get_widget("/ToolBar")
        tbar.set_tooltips(True)
        table.attach(tbar,
            # X direction #       # Y direction
            0, 1,                   1, 2,
            gtk.EXPAND | gtk.FILL,  0,
            0,                      0)

        # Create canvas
        self.sw = gtk.ScrolledWindow()
        self.sw.add_with_viewport(self.canvas)
        self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.sw.set_shadow_type(gtk.SHADOW_IN)

        table.attach(self.sw,
            # X direction           Y direction
            0, 1,                   2, 3,
            gtk.EXPAND | gtk.FILL,  gtk.EXPAND | gtk.FILL,
            0,                      0)

        self.canvas.grab_focus()

        # Create statusbar
        self.statusbar = gtk.Statusbar()
        table.attach(self.statusbar,
            # X direction           Y direction
            0, 1,                   3, 4,
            gtk.EXPAND | gtk.FILL,  0,
            0,                      0)

        # Show text widget info in the statusbar
        #buffer = contents.get_buffer()
        #buffer.connect("changed", self.update_statusbar)
        #mark_set_callback = (lambda buffer, new_location, mark:
        #    self.update_statusbar(buffer))

        # cursor moved
        #buffer.connect("mark_set", mark_set_callback)

        #self.connect("window_state_event", self.update_resize_grip)
        #self.update_statusbar(buffer)

        self.show_all()

        self.lab_filename = None    # the filename from which a lab is loaded

    def __quit(self, *args):
        """Quit the application"""
        running = False
        for d in lab.devices:
            if d.started:
                running = True
        if running is True:
            msg = "Warning, some devices are still running. Really quit?"
            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
                            gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO,
                            msg)
            dialog.set_title('Warning!')
            result = dialog.run()
            dialog.destroy()
            if result == gtk.RESPONSE_YES:
                gtk.main_quit()
                return False
            else:
                return True
        gtk.main_quit()
        return False

    def __edit_menu_activated(self, widget=None):
        """Called when the edit menu is activated"""
        self.uimanager.remove_action_group(self.add_action_group)
        self.add_action_group = self.__create_add_action_group()
        self.uimanager.insert_action_group(self.add_action_group, 0)

    def __create_action_group(self):
        # GtkActionEntry
        entries = (
          ("FileMenu", None, "_File" ),               # name, stock id, label
          ("EditMenu", None, "_Edit", None, None, self.__edit_menu_activated),
          ("DevicesMenu", None, "_Devices" ),
          ("ToolsMenu", None, "_Tools" ),
          ("PreferencesMenu", None, "_Preferences" ), # name, stock id, label
          ("HelpMenu", None, "_Help" ),               # name, stock id, label
          ("New", gtk.STOCK_NEW,                      # name, stock id
           "_New", "<control>N",                      # label, accelerator
           "Create a new file",                       # tooltip
           self.new_lab ),
          ("Open", gtk.STOCK_OPEN,                    # name, stock id
           "_Open","<control>O",                      # label, accelerator
           "Open a file",                             # tooltip
           self.open_lab ),
          ("Save", gtk.STOCK_SAVE,                    # name, stock id
           "_Save","<control>S",                      # label, accelerator
           "Save current file",                       # tooltip
           self.save_lab ),
          ("SaveAs", gtk.STOCK_SAVE,                  # name, stock id
           "Save _As...", None,                       # label, accelerator
           "Save to a file",                          # tooltip
           self.save_as_lab ),
          ("Quit", gtk.STOCK_QUIT,                    # name, stock id
           "_Quit", "<control>Q",                     # label, accelerator
           "Quit",                                    # tooltip
           self.__quit ),
          ("StartAll", None,                          # name, stock id
           "Start All", None,                         # label, accelerator
           "Start all devices",                       # tooltip
           self.canvas.start_all ),
          ("StopAll", None,                           # name, stock id
           "Stop All", None,                          # label, accelerator
           "Stop all devices",                        # tooltip
           self.canvas.stop_all),
          ("SuspendAll", None,                        # name, stock id
           "Suspend All", None,                       # label, accelerator
           "Suspend all devices",                     # tooltip
           self.canvas.suspend_all ),
          ("ResumeAll", None,                         # name, stock id
           "Resume All", None,                        # label, accelerator
           "Resume all devices",                      # tooltip
           self.canvas.resume_all ),
          ("OutputConfig", None,
           "Output Config", None,
           "Output saved configuration to files",
           self.output_config ),
          ("About", None,                             # name, stock id
           "_About", "<control>A",                    # label, accelerator
           "About",                                   # tooltip
           self.activate_about ),
        );

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("AppWindowActions")
        action_group.add_actions(entries)

        return action_group

    def __create_add_action_group(self, x=None, y=None):
        """Creates an action group for the 'add' actions"""
        entries = (
          ( "Add7200", None,                           # name, stock id
            "Add a _7200 router", "<control>7",        # label, accelerator
            "Add a new 7200 router",                   # tooltip
            self.add_router ),
          ( "Add3600", None,                           # name, stock id
            "Add a _3600 router", "<control>3",        # label, accelerator
            "Add a new 3600 router",                   # tooltip
            self.add_router ),
          ( "AddFRSwitch", None,                       # name, stock id
            "Add _FR Switch", "<control>F",            # label, accelerator
            "Add a new frame-relay switch",            # tooltip
            self.add_fr_switch ),
          ( "AddEthSwitch", None,                      # name, stock id
            "Add _Ethernet Switch", "<control>E",      # label, accelerator
            "Add a new Ethernet switch",               # tooltip
            self.add_eth_switch ),
          ( "AddATMSwitch", None,                      # name, stock id
            "Add _ATM Switch", "<control>A",           # label, accelerator
            "Add a new ATM switch",                    # tooltip
            self.add_atm_switch ),
        );
        # Create action group and pass coordinates (x, y) if exists
        action_group = gtk.ActionGroup("AppWindowAddActions")
        if x is not None and y is not None:
            action_group.add_actions(entries, (x, y))
        else:
            action_group.add_actions(entries)
        return action_group


    def add_router(self, widget, x=None, y=None):
        """
        Add a new 7200 router to the canvas
        x, y gives the position at which the router should be positionned.
        """
        if widget.get_name() == 'Add7200':
            rpd = C7200PropertiesDialog(self)
        elif widget.get_name() == 'Add3600':
            rpd = C3600PropertiesDialog(self)
        r = rpd.router
        if r is not None:
            r.init_draw(self.window.cairo_create())
            if x is not None and y is not None:
                r.px = r.x = x
                r.py = r.y = y
            lab.devices.append(r)
            self.canvas.redraw_canvas(r.dirty_area())
            self.update_devices_menu_list(r)

    def update_devices_menu_list(self, router):
        # update the menu list
        ui1 = '''<ui>
<menubar name='MenuBar'>
<menu action='DevicesMenu'>
  <menuitem action='%s'/>
</menu>
</menubar>
</ui>''' % router.name
        ac1 = gtk.ActionGroup('UIMerge')
        ac1.add_actions([((router.name, None, router.name, None,
                           'Access '+router.name, self.connect_device))])
        self.uimanager.insert_action_group(ac1, 1)
        id = self.uimanager.add_ui_from_string(ui1)

    def add_atm_switch(self, widget, x=None, y=None):
        """
        Add a new ATM to the canvas
        x, y gives the position at which the router should be positionned.
        """
        apd = ATMSwitchPropertiesDialog(self)
        r = apd.atmswitch
        if r is not None:
            r.init_draw(self.window.cairo_create())
            if x is not None and y is not None:
                r.px = r.x = x
                r.py = r.y = y
            lab.devices.append(r)
            self.canvas.redraw_canvas(r.dirty_area())

    def add_fr_switch(self, widget, x=None, y=None):
        """
        Add a new Frame Relay Switch to the canvas
        x, y gives the position at which the router should be positionned.
        """
        fpd = FRSwitchPropertiesDialog(self)
        r = fpd.frswitch
        if r is not None:
            r.init_draw(self.window.cairo_create())
            if x is not None and y is not None:
                r.px = r.x = x
                r.py = r.y = y
            lab.devices.append(r)
            self.canvas.redraw_canvas(r.dirty_area())

    def add_eth_switch(self, widget, x=None, y=None):
        """Add a new Ethernet Switch to the canvas
        x, y gives the position at which the router should be positionned.
        """
        epd = EthSwitchPropertiesDialog(self)
        r = epd.ethswitch
        if r is not None:
            r.init_draw(self.window.cairo_create())
            if x is not None and y is not None:
                r.px = r.x = x
                r.py = r.y = y
            lab.devices.append(r)
            self.canvas.redraw_canvas(r.dirty_area())

    def connect_device(self, action):
        """Connect to a started device"""
        n = action.get_name()
        for d in lab.devices:
            if d.name == n:
                self.canvas.connect_device(d)
                return

    def activate_about(self, action):
        """Display the About dialog"""
        dialog = gtk.AboutDialog()
        dialog.set_name("Dynagui v%s" % __version__)
        dialog.set_copyright("\302\251 Copyright 2006, 2007 Yannick Le Teigner")
        dialog.set_website("http://dynagui.sourceforge.net/")
        ## Close dialog on user response
        dialog.connect ("response", lambda d, r: d.destroy())
        dialog.show()

    def _build_popup_menu(self, x=None, y=None):
        """Build the menu appearing on the canvas on a right click"""
        if x is not None and y is not None:
            x, y = int(x), int(y)
        self.uimanager.remove_action_group(self.add_action_group)
        self.add_action_group = self.__create_add_action_group(x, y)
        self.uimanager.insert_action_group(self.add_action_group, 0)
        self.uimanager.add_ui_from_string(ui_popup)
        return self.uimanager.get_widget("/EditMenu")


    def new_lab(self, widget):
        """Opens the frame to add a new dynamips server"""
        # FIXME should clean current lab
        DynamipsPropertiesDialog(self, None)

    def open_lab(self, widget):
        """Opens the dialog to open an existing lab"""
        chooser = gtk.FileChooserDialog(title=None,
                                        action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)
        filter = gtk.FileFilter()
        filter.set_name("Network files")
        filter.add_pattern("*.net")
        chooser.add_filter(filter)
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            self.open_lab_file(chooser.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no files selected'
        chooser.destroy()
        #chooser.set_current_folder(pathname)

    def open_lab_file(self, filename):
        self.lab_filename = filename
        lab_io.load_lab(self.lab_filename)#, use_running=True)
        self.canvas.redraw_canvas()
        self.set_title("Dynagui - %s"
                    % os.path.basename(self.lab_filename))
        max_x = max_y = 0
        for d in lab.devices:
            d.init_draw(self.window.cairo_create())
            dis, dis, x, y = d.dirty_area()
            if x > max_x:
                max_x = x
            if y > max_y:
                max_y = y
            if isinstance(d, GuiRouter):
                self.update_devices_menu_list(d)
        (x, y) =  self.canvas.get_size_request()
        self.canvas.set_size_request(max_x > x and max_x or x,
                                     max_y > y and max_y or y)

    def save_as_lab(self, widget):
        """Opens the dialog to save the current lab in a file"""
        chooser = gtk.FileChooserDialog(title=None,
                                    action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=(gtk.STOCK_CANCEL,
                                             gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_OPEN,
                                             gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)
        filter = gtk.FileFilter()
        filter.set_name("Network files")
        filter.add_pattern("*.net")
        chooser.add_filter(filter)
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            self.lab_filename = chooser.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no files selected'
        chooser.destroy()
        if response == gtk.RESPONSE_OK:
            lab_io.save_lab(self.lab_filename)
            self.set_title("Dynagui - %s"
                        % os.path.basename(self.lab_filename))

    def save_lab(self, widget):
        """Save the lab in current lab file, if current lab file does
        not exist, opens a dialog to select lab file"""
        if self.lab_filename is not None:
            lab_io.save_lab(self.lab_filename)
        else:
            chooser = gtk.FileChooserDialog(title=None,
                                        action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK))
            chooser.set_default_response(gtk.RESPONSE_OK)
            filter = gtk.FileFilter()
            filter.set_name("Network files")
            filter.add_pattern("*.net")
            chooser.add_filter(filter)
            response = chooser.run()
            if response == gtk.RESPONSE_OK:
                self.lab_filename = chooser.get_filename()
                lab_io.save_lab(self.lab_filename)
                self.set_title("Dynagui - %s"
                        % os.path.basename(self.lab_filename))
            elif response == gtk.RESPONSE_CANCEL:
                print 'Closed, no files selected'
            chooser.destroy()

    def output_config(self, widget):
        """Save routers' configuration (as it is in the lab file) to a
        specific folder"""

        if len(lab.devices)>0:
            text = 'Select routers from which to extract configuration'
            rcd = RouterSelectionDialog(lab.devices, text, show_path=True)
            from dynagui_libs.nvram_io import read_conf_b64gz
            for d in rcd.selected_devices:
                cgz64 = d.config_gz64
                conf = read_conf_b64gz(cgz64)
                file = open(rcd.path+os.sep+d.name+'.txt', 'w')
                file.write(conf)
                file.close()



def main(args):
    d = DynaguiFrame()
    if len(args)>1:
        d.open_lab_file(args[1])
    gtk.main()
    return 0

import dynamips_lib
if not dynamips_lib.version.startswith('0.10.1'):
    print "Please use version 0.10.1 of dynagen"
    sys.exit()

if __name__ == "__main__":
    from dynamips_lib import BaseAdapter
    from dynamips_lib import NIO_udp
    from dynamips_lib import send
    # needed to patch BaseAdapter for OIR
    def remove(self):
        #if the router is running let's send also the OIR event.
        #"c7200 remove_pa_binding <instance_name> <slot>"
        #"c7200 pa_stop_online <instance_name> <slot>"
        # Code copied from Pavel Skovajsa's confDynagen
        if self._BaseAdapter__router.state == 'running':
            if  self._BaseAdapter__router.model == 'c7200':
                send(self._BaseAdapter__router.dynamips, '%s pa_stop_online %s %i' % (self._BaseAdapter__router.model, self._BaseAdapter__router.name, self.slot))
        #remove the PA from the router
        if self._BaseAdapter__router.model == 'c7200':
            send(self._BaseAdapter__router.dynamips, '%s remove_pa_binding %s %i' % (self._BaseAdapter__router.model, self._BaseAdapter__router.name, self.slot))
        else:
            send(self._BaseAdapter__router.dynamips, '%s remove_nm_binding %s %i' % (self._BaseAdapter__router.model, self._BaseAdapter__router.name, self.slot))

    BaseAdapter.remove = remove

    # needed to delete an nio_binding
    def delete_nio(self, port):
        send(self._BaseAdapter__router.dynamips, '%s remove_nio_binding %s %i %i' % (self._BaseAdapter__router.model, self._BaseAdapter__router.name, self.slot, port))
        self._BaseAdapter__nios[port].delete()

    BaseAdapter.delete_nio = delete_nio

    # needed to delete an nio
    def delete(self):
        send(self._NIO_udp__d, 'nio delete %s' % (self._NIO_udp__name))

    NIO_udp.delete = delete


    main(sys.argv)

