#!/usr/bin/env python

"""
C3600_properties_dialog.py
Copyright (C) 2006, 2007 Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import os
import gtk
import gobject
from gui_c3600 import GuiC3600, GuiLeopard_2FE, GuiNM_1FE_TX, GuiNM_1E, GuiNM_4E, GuiNM_4T, GuiNM_16ESW
from gui_router import dynamips
from dynamips_properties_dialog import DynamipsPropertiesDialog
from router_properties_dialog import RouterPropertiesDialog
from lab import lab

class C3600PropertiesDialog(RouterPropertiesDialog):
    slot_num = 7

    def __init__(self, parent, router=None):
        self.parent = parent
        self.router = router
        self.__build_gui()
        self.__fill_default()
        # run the dialog
        ret = self.w.run()
        self.w.hide()


    def __fill_default(self):
        """Set some default values based on the router.
        """
        if self.router is None:
            if len(dynamips)>0:
                self.combo_dynamips.set_active(0)
            self.entry_ram.set_text("128")
            self.combo_chassis.set_active(0)
            #for i in range(1, 7):
            #    self.combo_pa[i].set_active(0)
            self.button_color.set_color(
                        gtk.gdk.Color(int(0.4*65535),
                                      int(0.6*65535),
                                      int(0.6*65535)))
        else:
            self.slot_num = len(self.router.slot)
            self.entry_name.set_sensitive(False)
            for i,d in enumerate(dynamips):
                if d.host==self.router.dynamips.host \
                    and d.port==self.router.dynamips.port:
                    self.combo_dynamips.set_active(i)
            if self.router.name is not None:
                self.entry_name.set_text(str(self.router.name))
            for i,v in enumerate(self.chassisList):
                if v==self.router.chassis:
                    self.combo_chassis.set_active(i)
            self.combo_chassis.set_sensitive(False)
            if self.router.ram is not None:
                self.entry_ram.set_text(str(self.router.ram))
            self.check_sparsemem.set_active(self.router.sparsemem)
            if self.router.console is not None:
                self.entry_console.set_text(str(self.router.console))
            if self.router.idlepc is not None:
                self.entry_idlepc.set_text(str(self.router.idlepc))
            if self.router.image is not None:
                self.button_ios.set_filename(self.router.image)
            active_pa = [0] * self.slot_num
            for i,v in enumerate(self.pa_list):
                for j in range(0, self.slot_num):
                    if self.router.slot[j]!=None \
                            and self.router.slot[j].adapter==v:
                        active_pa[j] = i
            self.button_color.set_color(self.router.gdkcolor)
            # FIXME 3660
            #for i,v in enumerate(self.paio_list):
            #    if self.router.slot[0]!=None \
            #            and self.router.slot[0].adapter==v:
            #        active_pa[0] = i
            for i in range(0, self.slot_num):
                self.combo_pa[i].set_active(active_pa[i])
            # no OIR yet
            for i in range(0, self.slot_num):
                if self.router.slot[i] is not None:
                    self.combo_pa[i].set_sensitive(False)

            # disable some parameters when router is on
            if self.router.started:
                self.combo_chassis.set_sensitive(False)
                self.entry_ram.set_sensitive(False)
                self.check_sparsemem.set_sensitive(False)
                self.button_ios.set_sensitive(False)

        # prefill the ios folder
        # FIXME needs to prefill the ios if the router type is the same
        # to be done dynamically with an event listener
        # if dynamips != null
        # select the last router with same model, and same dynamips
        # and put the same ios/idle-pc
        dynamips_text = self.combo_dynamips.get_active_text()
        self.ios_folder = None
        if dynamips_text is not None:
            d_host, d_port = dynamips_text.split(':')
            for d in lab.devices:
                if d.dynamips.host == d_host and d.dynamips.port == int(d_port):
                    self.ios_folder = d.image[:d.image.rfind(os.sep)]
                    #if self.ios_folder is not None:
                    self.button_ios.set_current_folder(self.ios_folder)
                    break


    def __build_gui(self):
        self.w = gtk.Dialog("Router properties", self.parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        #self.w.set_size_request(300, 300)
        # the main content
        #self.w.connect("destroy", lambda w: gtk.main_quit())
        self.table_layout = gtk.Table(rows=10, columns=2)
        self.table_layout.set_row_spacings(5)
        self.table_layout.set_col_spacings(5)

        # the dynamips entry
        label_dynamips = gtk.Label('Dynamips: ')
        self.list_dynamips = gtk.ListStore(str)
        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        self.combo_dynamips = gtk.ComboBox(self.list_dynamips)
        cell = gtk.CellRendererText()
        self.combo_dynamips.pack_start(cell, True)
        self.combo_dynamips.add_attribute(cell, 'text', 0)
        button_edit_dynamips = gtk.Button(stock=gtk.STOCK_ADD)
        button_edit_dynamips.connect("clicked", self.edit_dynamips, gtk.RESPONSE_OK)

        # the name entry
        label_name = gtk.Label('Name: ')
        self.entry_name = gtk.Entry()
        self.entry_name.set_max_length(15)
        self.entry_name.set_width_chars(8)

        # chassis type
        label_chassis = gtk.Label('Chassis: ')
        list_store = gtk.ListStore(str)
        self.chassisList = ['3620', '3640', '3660']
        for i,v in enumerate(self.chassisList):
            list_store.append([v])
        self.combo_chassis = gtk.ComboBox(list_store)
        cell = gtk.CellRendererText()
        self.combo_chassis.pack_start(cell, True)
        self.combo_chassis.add_attribute(cell, 'text', 0)
        self.combo_chassis.connect("changed", self.chassis_changed)

        # the RAM entry
        label_ram = gtk.Label('RAM: ')
        self.entry_ram = gtk.Entry()
        self.entry_ram.set_max_length(3)
        self.entry_ram.set_width_chars(4)

        # the sparsemem option
        label_sparsemem = gtk.Label('Sparse mem: ')
        self.check_sparsemem = gtk.CheckButton()

        # the console port entry
        label_console = gtk.Label('Console: ')
        self.entry_console = gtk.Entry()
        self.entry_console.set_max_length(5)
        self.entry_console.set_width_chars(4)


        # the ios selector
        label_ios = gtk.Label('IOS: ')
        self.button_ios = gtk.FileChooserButton('Select an IOS')
        self.button_ios.set_width_chars(20)
        self.button_ios.connect("selection-changed", self.update_idlepc, None)

        # the Idle PC entry
        label_idlepc = gtk.Label('Idle PC: ')
        self.entry_idlepc = gtk.Entry()
        #self.entry_idlepc.set_max_length(10)
        self.entry_idlepc.set_width_chars(10)

        # the background color
        label_color = gtk.Label('Background color: ')
        self.button_color = gtk.ColorButton()

        ypos = 0
        label_dynamips.set_alignment(0, 0.5)
        table_dyn = gtk.Table(rows=1, columns=2)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.combo_dynamips)
        al1 = gtk.Alignment()
        al1.set(0.1, 0.0, 0.0, 1.0)
        al1.add(button_edit_dynamips)
        table_dyn.attach(al, 0, 1, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        table_dyn.attach(al1, 1, 2, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        self.table_layout.attach(label_dynamips, 0, 1, ypos, ypos+1)
        self.table_layout.attach(table_dyn, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_name.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.entry_name)
        self.table_layout.attach(label_name, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_chassis.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.combo_chassis)
        self.table_layout.attach(label_chassis, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_ram.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_ram)
        self.table_layout.attach(label_ram, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_sparsemem.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.check_sparsemem)
        self.table_layout.attach(label_sparsemem, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_console.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_console)
        self.table_layout.attach(label_console, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)


        ypos += 1
        label_ios.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.button_ios)
        self.table_layout.attach(label_ios, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            yoptions=gtk.EXPAND|gtk.FILL)

        ypos += 1
        label_idlepc.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_idlepc)
        self.table_layout.attach(label_idlepc, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_color.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.button_color)
        self.table_layout.attach(label_color, 0, 1, ypos, ypos+1)
        self.table_layout.attach(al, 1, 2, ypos, ypos+1)


        self.w.vbox.pack_start(self.table_layout)
        # the button panel
        button_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
        button_ok = gtk.Button(stock=gtk.STOCK_OK)
        self.w.action_area.pack_start(button_cancel)
        self.w.action_area.pack_start(button_apply)
        self.w.action_area.pack_start(button_ok)
        button_cancel.connect("clicked", self.clicked, gtk.RESPONSE_CANCEL)
        button_apply.connect("clicked", self.clicked, gtk.RESPONSE_APPLY)
        button_ok.connect("clicked", self.clicked, gtk.RESPONSE_OK)

        self.w.show_all()

    def close(self, widget, event):
        w.response(0)

    def edit_dynamips(self, widget, data):
        DynamipsPropertiesDialog(self.parent, None)
        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        #self.list_dynamips = [["%s:%d" % (d.host, d.port)] for d in dynamips]
        print self.list_dynamips
        self.combo_dynamips.set_active(0)

    def d_error(self, msg):
        """ Display an error message"""
        dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                            gtk.BUTTONS_OK, msg)
        dialog.run()
        dialog.destroy()

    def chassis_changed(self, widget):
        chassis = self.combo_chassis.get_active_text()
        if chassis == '3620':
            panum = 2
        elif chassis == '3640':
            panum = 4
        else:
            panum = 6
        # the PA list
        self.pa_list = ['', 'NM-1E', 'NM-4E', 'NM-1FE-TX', 'NM-4T', 'NM-16ESW']
        pa_store = gtk.ListStore(str)
        for i,v in enumerate(self.pa_list):
            pa_store.append([v])

        try:
            self.table_pa.destroy()
        except:
            pass
        self.table_pa = gtk.Table(rows=4, columns=4)
        self.table_pa.set_row_spacings(5)
        self.table_pa.set_col_spacings(5)
        self.combo_pa = []
        sindex = 0
        if panum==6:
            # pa 0 / IO adapter
            self.paio_list = ['Leopard_2FE']
            paio_store = gtk.ListStore(str)
            for i,v in enumerate(self.paio_list):
                paio_store.append([v])
            self.combo_pa.append(gtk.ComboBox(paio_store))
            self.combo_pa[0].set_sensitive(False)   # cannot change it
            self.combo_pa[0].show()
            cell = gtk.CellRendererText()
            self.combo_pa[0].pack_start(cell, True)
            self.combo_pa[0].add_attribute(cell, 'text', 0)
            self.combo_pa[0].set_active(0) #FIXME
            button_pa = gtk.Label('Slot 0')
            button_pa.show()
            self.table_pa.attach(button_pa, 0, 1, 3, 4)
            self.table_pa.attach(self.combo_pa[0], 1, 4, 3, 4)
            sindex = 1

        ypos = panum/2
        for i in range(sindex, panum+sindex):
            self.combo_pa.append(gtk.ComboBox(pa_store))
            self.combo_pa[i].show()
            cell = gtk.CellRendererText()
            self.combo_pa[i].pack_start(cell, True)
            self.combo_pa[i].add_attribute(cell, 'text', 0)
            self.combo_pa[i].set_active(0) # FIXME or pre-existing

            button_pa = gtk.Label('Slot %s:' % i)
            button_pa.show()
            if i%2==sindex:
                l, r = 0, 1
            else:
                l, r = 2, 3
            self.table_pa.attach(button_pa, l, r, ypos-1, ypos)
            self.table_pa.attach(self.combo_pa[i], l+1, l+2, ypos-1, ypos)
            if i%2==(not sindex):
                ypos -= 1

        self.table_layout.attach(self.table_pa, 0, 2, 9, 10)
        self.table_layout.show_all()



    def clicked(self, widget, data):
        if data==gtk.RESPONSE_APPLY or data==gtk.RESPONSE_OK:
            if self.router is None:
                dynamips_text = self.combo_dynamips.get_active_text()
                if dynamips_text is None:
                    self.d_error("Please select a dynamips instance")
                    return
                name = self.entry_name.get_text()
                if name is None or name == "":
                    self.d_error("Please select a router name")
                    return
                for d in dynamips:
                    if ("%s:%d" % (d.host, d.port))==dynamips_text:
                        chassis = self.combo_chassis.get_active_text()
                        self.router = GuiC3600(dynamips=d, name=name,
                                               chassis=chassis)
                        break

            image = self.button_ios.get_filename()
            if image is not None:
                self.router.image = image

            self.router.ram = int(self.entry_ram.get_text())
            self.router.sparsemem = self.check_sparsemem.get_active()
            try:
                cons = int(self.entry_console.get_text())
                self.router.console = cons
            except:
                pass
            if self.entry_idlepc.get_text() != '':
                self.router.idlepc = self.entry_idlepc.get_text()
            c = self.button_color.get_color()
            self.router.chassis = self.combo_chassis.get_active_text()
            self.router.set_bgcolor(c.red/65535.0, c.green/65535.0,
                            c.blue/65535.0)
            for i in range(0, len(self.combo_pa)):
                if self.router.slot[i] is None: # no OIR yet
                    index = self.combo_pa[i].get_active()
                    if self.pa_list[index]=='NM-1E':
                        self.router.slot[i] = GuiNM_1E(self.router, i)
                    elif self.pa_list[index]=='NM-4E':
                        self.router.slot[i] = GuiNM_4E(self.router, i)
                    elif self.pa_list[index]=='NM-1FE-TX':
                        self.router.slot[i] = GuiNM_1FE_TX(self.router, i)
                    elif self.pa_list[index]=='NM-4T':
                        self.router.slot[i] = GuiNM_4T(self.router, i)
                    elif self.pa_list[index]=='NM-16ESW':
                        self.router.slot[i] = GuiNM_16ESW(self.router, i)
        if data==gtk.RESPONSE_CANCEL or data==gtk.RESPONSE_OK:
            self.w.destroy()
