#!/usr/bin/env python

"""
C7200_properties_dialog.py
Copyright (C) 2006, 2007 Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
import gobject
from gui_c7200 import GuiC7200, PA, GuiPA_FE_TX, GuiPA_4E, GuiPA_8E, GuiPA_4Tp, GuiPA_8T, GuiPA_A1, GuiC7200_IO_FE, GuiC7200_IO_2FE
from gui_router import dynamips, GuiRouter
from dynamips_lib import Dynamips
from dynamips_properties_dialog import DynamipsPropertiesDialog
from router_properties_dialog import RouterPropertiesDialog
from lab import lab

class C7200PropertiesDialog(RouterPropertiesDialog):
    slot_num = 7

    pa_type = {'PA-FE-TX': GuiPA_FE_TX,
               'PA-4E': GuiPA_4E,
               'PA-8E': GuiPA_8E,
               'PA-4T+': GuiPA_4Tp,
               'PA-8T': GuiPA_8T,
               'PA-A1': GuiPA_A1}

    paio_type = {'C7200-IO-FE': GuiC7200_IO_FE,
                 'C7200-IO-2FE': GuiC7200_IO_2FE}

    def __init__(self, parent, router=None):
        self.parent = parent
        self.router = router
        self.__build_gui()
        self.__fill_default()
        # run the dialog
        ret = self.w.run()
        self.w.hide()


    def __fill_default(self):
        """Set some default values based on the router.
        """
        if self.router is None:
            if len(dynamips)>0:
                self.combo_dynamips.set_active(0)
                for d in lab.devices:
                    if isinstance(d, GuiRouter) and d.dynamips == dynamips[0]:
                        # preset the IOS to a similar router instance's image
                        if d.image is not None:
                            self.button_ios.set_filename(d.image)
                        break

            self.entry_ram.set_text("128")
            self.combo_npe.set_active(3)
            self.combo_midplane.set_active(1)
            for i in range(1, 7):
                self.combo_pa[i].set_active(0)
            self.button_color.set_color(
                        gtk.gdk.Color(int(0.4*65535),
                                      int(0.6*65535),
                                      int(0.6*65535)))

        else:
            self.entry_name.set_sensitive(False)
            for i,d in enumerate(dynamips):
                if d.host==self.router.dynamips.host \
                    and d.port==self.router.dynamips.port:
                    self.combo_dynamips.set_active(i)
            if self.router.name is not None:
                self.entry_name.set_text(str(self.router.name))
            for i,v in enumerate(self.npeList):
                if v==self.router.npe:
                    self.combo_npe.set_active(i)
            for i,v in enumerate(self.midplane_list):
                if v==self.router.midplane:
                    self.combo_midplane.set_active(i)
            if self.router.ram is not None:
                self.entry_ram.set_text(str(self.router.ram))
            self.check_sparsemem.set_active(self.router.sparsemem)
            if self.router.console is not None:
                self.entry_console.set_text(str(self.router.console))
            if self.router.idlepc is not None:
                self.entry_idlepc.set_text(str(self.router.idlepc))
            if self.router.image is not None:
                self.button_ios.set_filename(self.router.image)
            active_pa = [0] * self.slot_num
            for i,v in enumerate(self.pa_list):
                for j in range(1, self.slot_num):
                    if self.router.slot[j]!=None \
                            and self.router.slot[j].adapter==v:
                        active_pa[j] = i
            self.button_color.set_color(self.router.gdkcolor)
            for i,v in enumerate(self.paio_list):
                if self.router.slot[0]!=None \
                        and self.router.slot[0].adapter==v:
                    active_pa[0] = i
                    self.combo_pa[0].set_active(active_pa[0])
            for i in range(1, 7):
                self.combo_pa[i].set_active(active_pa[i])

            # disable some parameters when router is started
            if self.router.started:
                self.combo_npe.set_sensitive(False)
                self.combo_midplane.set_sensitive(False)
                self.entry_ram.set_sensitive(False)
                self.check_sparsemem.set_sensitive(False)
                self.button_ios.set_sensitive(False)
                # no OIR yet
                for i in range(0, 7):
                    if self.router.slot[i] is not None:
                        self.combo_pa[i].set_sensitive(False)



    def __build_gui(self):
        self.w = gtk.Dialog("Router properties", self.parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        #self.w.set_size_request(300, 300)
        # the main content
        #self.w.connect("destroy", lambda w: gtk.main_quit())
        table_layout = gtk.Table(rows=9, columns=2)
        table_layout.set_row_spacings(5)
        table_layout.set_col_spacings(5)

        # the dynamips entry
        label_dynamips = gtk.Label('Dynamips: ')
        self.list_dynamips = gtk.ListStore(str)

        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        self.combo_dynamips = gtk.ComboBox(self.list_dynamips)
        cell = gtk.CellRendererText()
        self.combo_dynamips.pack_start(cell, True)
        self.combo_dynamips.add_attribute(cell, 'text', 0)
        button_edit_dynamips = gtk.Button(stock=gtk.STOCK_ADD)
        button_edit_dynamips.connect("clicked", self.edit_dynamips, gtk.RESPONSE_OK)

        # the name entry
        label_name = gtk.Label('Name: ')
        self.entry_name = gtk.Entry()
        self.entry_name.set_max_length(15)
        self.entry_name.set_width_chars(8)

        # npe type
        label_npe = gtk.Label('NPE: ')
        list_store = gtk.ListStore(str)
        self.npeList = ['npe-100', 'npe-150', 'npe-175', 'npe-200', 'npe-225',
                   'npe-300', 'npe-400']
        for i,v in enumerate(self.npeList):
            list_store.append([v])
        self.combo_npe = gtk.ComboBox(list_store)
        cell = gtk.CellRendererText()
        self.combo_npe.pack_start(cell, True)
        self.combo_npe.add_attribute(cell, 'text', 0)

        # midplane
        label_midplane = gtk.Label('Midplane: ')
        midplane_list_store = gtk.ListStore(str)
        self.midplane_list = ['std', 'vxr']
        for i,v in enumerate(self.midplane_list):
            midplane_list_store.append([v])
        self.combo_midplane = gtk.ComboBox(midplane_list_store)
        cell = gtk.CellRendererText()
        self.combo_midplane.pack_start(cell, True)
        self.combo_midplane.add_attribute(cell, 'text', 0)

        # the RAM entry
        label_ram = gtk.Label('RAM: ')
        self.entry_ram = gtk.Entry()
        self.entry_ram.set_max_length(3)
        self.entry_ram.set_width_chars(4)

        # the sparsemem option
        label_sparsemem = gtk.Label('Sparse mem: ')
        self.check_sparsemem = gtk.CheckButton()

        # the console port entry
        label_console = gtk.Label('Console: ')
        self.entry_console = gtk.Entry()
        self.entry_console.set_max_length(5)
        self.entry_console.set_width_chars(4)


        # the ios selector
        label_ios = gtk.Label('IOS: ')
        self.button_ios = gtk.FileChooserButton('Select an IOS')
        self.button_ios.set_width_chars(20)
        # to be done dynamically with an event listener
        # if dynamips != null
        # select the last router with same model, and same dynamips
        # and put the same ios/idle-pc
        #if self.ios_folder is not Null: #FIXME
        #    filechooserbutton.set_current_folder(self.ios_folder)
        self.button_ios.connect("selection-changed", self.update_idlepc, None)

        # the Idle PC entry
        label_idlepc = gtk.Label('Idle PC: ')
        self.entry_idlepc = gtk.Entry()
        #self.entry_idlepc.set_max_length(10)
        self.entry_idlepc.set_width_chars(10)

        # the background color
        label_color = gtk.Label('Background color: ')
        self.button_color = gtk.ColorButton()

        # the PA list
        self.pa_list = ['', 'PA-FE-TX', 'PA-4E', 'PA-8E', 'PA-4T+', 'PA-8T', 'PA-A1']
        pa_store = gtk.ListStore(str)
        for i,v in enumerate(self.pa_list):
            pa_store.append([v])

        table_pa = gtk.Table(rows=4, columns=4)
        table_pa.set_row_spacings(5)
        table_pa.set_col_spacings(5)
        self.combo_pa = []
        # pa 0 / IO adapter
        self.paio_list = self.paio_type.keys()
        paio_store = gtk.ListStore(str)
        for i,v in enumerate(self.paio_list):
            paio_store.append([v])
        self.combo_pa.append(gtk.ComboBox(paio_store))
        #self.combo_pa[0].set_sensitive(False)   # no OIR on the IO-FE
        self.combo_pa[0].show()
        cell = gtk.CellRendererText()
        self.combo_pa[0].pack_start(cell, True)
        self.combo_pa[0].add_attribute(cell, 'text', 0)
        #self.combo_pa[0].set_active(0) #FIXME
        button_pa = gtk.Label('Slot 0')
        button_pa.show()
        table_pa.attach(button_pa, 0, 1, 3, 4)
        table_pa.attach(self.combo_pa[0], 1, 4, 3, 4)

        ypos = 3
        for i in range(1, 7):
            self.combo_pa.append(gtk.ComboBox(pa_store))
            self.combo_pa[i].show()
            cell = gtk.CellRendererText()
            self.combo_pa[i].pack_start(cell, True)
            self.combo_pa[i].add_attribute(cell, 'text', 0)

            button_pa = gtk.Label('Slot %s:' % i)
            button_pa.show()
            if i%2==1:
                l, r = 0, 1
            else:
                l, r = 2, 3
            table_pa.attach(button_pa, l, r, ypos-1, ypos)
            table_pa.attach(self.combo_pa[i], l+1, l+2, ypos-1, ypos)
            if i%2==0:
                ypos -= 1

        ypos = 0
        label_dynamips.set_alignment(0, 0.5)
        table_dyn = gtk.Table(rows=1, columns=2)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.combo_dynamips)
        al1 = gtk.Alignment()
        al1.set(0.1, 0.0, 0.0, 1.0)
        al1.add(button_edit_dynamips)
        table_dyn.attach(al, 0, 1, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        table_dyn.attach(al1, 1, 2, 0, 1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)
        table_layout.attach(label_dynamips, 0, 1, ypos, ypos+1)
        table_layout.attach(table_dyn, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_name.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.entry_name)
        table_layout.attach(label_name, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_npe.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 0.5, 0.0, 1.0)
        al.add(self.combo_npe)
        table_layout.attach(label_npe, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_midplane.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.combo_midplane)
        table_layout.attach(label_midplane, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_ram.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_ram)
        table_layout.attach(label_ram, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_sparsemem.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.check_sparsemem)
        table_layout.attach(label_sparsemem, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_console.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_console)
        table_layout.attach(label_console, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)


        ypos += 1
        label_ios.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.button_ios)
        table_layout.attach(label_ios, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            yoptions=gtk.EXPAND|gtk.FILL)

        ypos += 1
        label_idlepc.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.entry_idlepc)
        table_layout.attach(label_idlepc, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1,
                            xoptions=gtk.EXPAND|gtk.FILL,
                            yoptions=gtk.EXPAND)

        ypos += 1
        label_color.set_alignment(0, 0.5)
        al = gtk.Alignment()
        al.set(0.0, 1.0, 0.0, 1.0)
        al.add(self.button_color)
        table_layout.attach(label_color, 0, 1, ypos, ypos+1)
        table_layout.attach(al, 1, 2, ypos, ypos+1)

        ypos += 1
        table_layout.attach(table_pa, 0, 2, ypos, ypos+1)

        self.w.vbox.pack_start(table_layout)
        # the button panel
        button_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
        button_ok = gtk.Button(stock=gtk.STOCK_OK)
        self.w.action_area.pack_start(button_cancel)
        self.w.action_area.pack_start(button_apply)
        self.w.action_area.pack_start(button_ok)
        button_cancel.connect("clicked", self.clicked, gtk.RESPONSE_CANCEL)
        button_apply.connect("clicked", self.clicked, gtk.RESPONSE_APPLY)
        button_ok.connect("clicked", self.clicked, gtk.RESPONSE_OK)

        self.w.show_all()

    def close(self, widget, event):
        w.response(0)

    def edit_dynamips(self, widget, data):
        DynamipsPropertiesDialog(self.parent, None)
        for d in dynamips:
            self.list_dynamips.append(["%s:%d" % (d.host, d.port)])
        #self.list_dynamips = [["%s:%d" % (d.host, d.port)] for d in dynamips]
        self.combo_dynamips.set_active(0)

    def d_error(self, msg):
        """ Display an error message"""
        dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                            gtk.BUTTONS_OK, msg)
        dialog.run()
        dialog.destroy()


    def clicked(self, widget, data):
        if data==gtk.RESPONSE_APPLY or data==gtk.RESPONSE_OK:
            if self.router is None:
                dynamips_text = self.combo_dynamips.get_active_text()
                if dynamips_text is None:
                    self.d_error("Please select a dynamips instance")
                    return
                name = self.entry_name.get_text()
                if name is None or name == "":
                    self.d_error("Please select a router name")
                    return
                for d in dynamips:
                    if ("%s:%d" % (d.host, d.port))==dynamips_text:
                        self.router = GuiC7200(dynamips=d, name=name)
                        break

            image = self.button_ios.get_filename()
            if image is not None:
                self.router.image = image

            self.router.ram = int(self.entry_ram.get_text())
            try:
                cons = int(self.entry_console.get_text())
                self.router.console = cons
            except:
                pass
            self.router.sparsemem = self.check_sparsemem.get_active()
            if self.entry_idlepc.get_text() != '':
                self.router.idlepc = self.entry_idlepc.get_text()
            if self.router.npe != self.combo_npe.get_active_text():
                self.router.npe = self.combo_npe.get_active_text()
            if self.router.midplane != self.combo_midplane.get_active_text():
                self.router.midplane = self.combo_midplane.get_active_text()
            c = self.button_color.get_color()
            self.router.set_bgcolor(c.red/65535.0, c.green/65535.0,
                            c.blue/65535.0)
            #if self.router.slot[0] is None: # no OIR yet
            # Do not try to change slot 0 when router is started
            if not self.router.started:
                index = self.combo_pa[0].get_active()
                if self.paio_list[index]=='C7200-IO-FE':
                    if isinstance(self.router.slot[0], GuiC7200_IO_2FE):
                        self.router.slot[0].remove() # remove old PA
                        self.router.slot[0] = None
                    if self.router.slot[0] is None:
                        self.router.slot[0] = GuiC7200_IO_FE(self.router, 0)
                elif self.paio_list[index]=='C7200-IO-2FE':
                    if isinstance(self.router.slot[0], GuiC7200_IO_FE):
                        self.router.slot[0].remove() # remove old PA
                        self.router.slot[0] = None
                    if self.router.slot[0] is None:
                        self.router.slot[0] = GuiC7200_IO_2FE(self.router, 0)
                else:
                    self.router.slot[0] = None
            for i in range(1, len(self.combo_pa)):
                index = self.combo_pa[i].get_active()
                for type_s in self.pa_type.keys():
                    if self.pa_list[index]==type_s:
                        if self.router.slot[i] is None:
                            self.router.slot[i] = \
                                    self.pa_type[type_s](self.router, i)
                        elif not isinstance(self.router.slot[i], self.pa_type[type_s]):
                            self.router.slot[i].remove()
                            self.router.slot[i] = \
                                    self.pa_type[type_s](self.router, i)
        if data==gtk.RESPONSE_CANCEL or data==gtk.RESPONSE_OK:
            self.w.destroy()

