#!/usr/bin/env python

"""
canvas.py
Copyright (C) 2007  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import os
import sys
import gtk
import time
from gtk import gdk
import dynamips_lib
from dynagui_libs.lab import lab, Connection
from dynagui_libs.util import setup_ghost
from dynagui_libs.gui_c3600 import GuiC3600
from dynagui_libs.gui_c7200 import GuiC7200
from dynagui_libs.gui_router import GuiRouter
from dynagui_libs.gui_frsw import GuiFRSwitch
from dynagui_libs.gui_atmsw import GuiATMSwitch
from dynagui_libs.gui_ethsw import GuiEthSwitch
from dynagui_libs.C3600_properties_dialog import C3600PropertiesDialog
from dynagui_libs.C7200_properties_dialog import C7200PropertiesDialog
from dynagui_libs.frsw_properties_dialog import FRSwitchPropertiesDialog
from dynagui_libs.atmsw_properties_dialog import ATMSwitchPropertiesDialog
from dynagui_libs.ethsw_properties_dialog import EthSwitchPropertiesDialog

term = 'xterm'          # linux terminal
win_telnet = "telnet "  # windows terminal
for p in os.environ['PATH'].split(';'):
    if os.path.exists(os.path.join(p, "putty.exe")):
        win_telnet = "putty.exe -telnet " # if putty is in path
        break


class MyCanvas(gtk.DrawingArea):
    def __init__(self, parent):
        super(MyCanvas, self).__init__()
        self.pressed = False # is the mouse button pressed
        self.myparent = parent
        self.set_size_request(400, 300)
        self.pangolayout = self.create_pango_layout("")
        self.add_events(gtk.gdk.POINTER_MOTION_MASK |
                        gtk.gdk.POINTER_MOTION_HINT_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK |
                        gtk.gdk.BUTTON_PRESS_MASK)
        self.connect("expose_event", self.area_expose_cb)
        self.connect("button_press_event", self.press)
        self.connect("button_release_event", self.release)
        self.connect("motion_notify_event", self.motion_notify)


    def motion_notify(self, widget, event):
        if event.is_hint:
            x, y, state = event.window.get_pointer()
        else:
            x, y, state = event.x, event.y, event.state

        if self.pressed:
            if lab.device is not None:
                lab.device.moveto(int(event.x), int(event.y))
                self.redraw_canvas(lab.device.dirty_area())
                # increase the drawing area?
                nw = lab.device.x + lab.device.w
                nh = lab.device.y + lab.device.h
                cw, ch = self.get_size_request()
                if nw>cw:
                    if nh>ch:
                        self.set_size_request(nw, nh)
                    else:
                        self.set_size_request(nw, ch)
                elif nh>ch:
                    self.set_size_request(cw, nh)
                self.redraw_canvas(lab.device.dirty_area())
            elif lab.connection is not None:
                lab.connection.move1to(int(event.x), int(event.y))
                self.redraw_canvas(lab.connection.dirty_area())
        return True

    def connect_device(self, device):
        """Launch the standard telnet program to connect to a virtual device"""
        if sys.platform.startswith("linux"):
            os.system('%s -T %s -e telnet %s %d&'
                    % (term, device.name, device.dynamips.host, device.console))
        elif sys.platform.startswith("win"):
            import subprocess
            subprocess.Popen("start %s localhost %d" %
                    (win_telnet, device.console), shell=True)

    def press(self, widget, event=None):
        """ the mouse button is pressed """
        if event.type == gtk.gdk._2BUTTON_PRESS: # double click
            for device in lab.devices:
                if device.inside(event.x, event.y):
                    self.connect_device(device)
                    return
        elif event.button==1: # left button
            self.pressed=True
            for i in lab.devices:
                if i.inside_pa_port(event.x, event.y):
                    if not i.inside_empty_port(event.x, event.y):
                        break
                    lab.connection = Connection(int(event.x), int(event.y))
                    lab.connection.devicea = i
                    lab.connections.append(lab.connection)
                    lab.device = None
                    return
                elif i.inside(event.x, event.y):
                    lab.device = i
                    lab.device.setdelta(int(event.x), int(event.y))
                    lab.connection = None
                    return
            for cct in lab.connections:
                if cct.inside(event.x, event.y):
                    # unselect any previous cct
                    for cc in lab.selected_ccts:
                        cc.selected = False
                    lab.selected_ccts = []
                    # select the new cct
                    cct.selected = True
                    lab.selected_ccts.append(cct)
                    self.redraw_canvas()
                    return
            # didn't click on devices or connections
            lab.connection = None
            lab.device = None
            for cc in lab.selected_ccts:
                cc.selected = False
            lab.selected_ccts = []
            self.redraw_canvas()
        elif event.button==3: # right click
            for cct in lab.connections:
                if cct.inside(event.x, event.y):
                    # unselect any previous cct
                    for cc in lab.selected_ccts:
                        cc.selected = False
                    lab.selected_ccts = []
                    # select the new cct
                    cct.selected = True
                    lab.selected_ccts.append(cct)
                    self.redraw_canvas()
                    self.buildlinkpopupmenu(event)
                    return
            for device in lab.devices:
                if device.inside(event.x, event.y):
                    self.buildpopupmenu(event, device)
                    return
            # didn't click on devices or connections
            for cc in lab.selected_ccts:
                cc.selected = False
            lab.selected_ccts = []
            self.redraw_canvas()
            self.myparent._build_popup_menu(event.x, event.y).popup(None, None, None, event.button, event.time)


    def release(self, widget, event=None):
        """ the mouse button is released """
        self.pressed = False
        if lab.connection is not None:
            for device in lab.devices:
                if device.inside_pa_port(event.x, event.y):
                    if device!=lab.connection.devicea and \
                            device.inside_empty_port(event.x, event.y):
                        try:
                            lab.connection.deviceb = device
                            lab.connection.devicea.addconnection(
                                                        lab.connection, 0)
                            # add to the router's connection list
                            device.addconnection(lab.connection, 1)
                            lab.connection = None
                            return
                        except dynamips_lib.DynamipsError:
                            print 'error connecting'
                            break

            lab.connections.remove(lab.connection)
            lab.connection = None
            self.redraw_canvas()


    def show_device_properties(self, widget, device):
        """ Displays the properties dialog for the Frame-relay switch"""
        if isinstance(device, GuiC7200):
            C7200PropertiesDialog(self.myparent, device)
        elif isinstance(device, GuiC3600):
            C3600PropertiesDialog(self.myparent, device)
        elif isinstance(device, GuiFRSwitch):
            FRSwitchPropertiesDialog(self.myparent, device)
        elif isinstance(device, GuiATMSwitch):
            ATMSwitchPropertiesDialog(self.myparent, device)
        elif isinstance(device, GuiEthSwitch):
            EthSwitchPropertiesDialog(self.myparent, device)
        self.redraw_canvas(device.dirty_area())
        return gtk.TRUE


    def start_device(self, widget=None, device=None):
        # check for ghost IOS
        if isinstance(device, GuiRouter):
            if device.mmap:
                setup_ghost(device)
            if device.gui_start():
                self.redraw_canvas(device.dirty_area())

    def stop_device(self, widget=None, device=None):
        if isinstance(device, GuiRouter) and device.gui_stop():
            r = device.dirty_area()
            # trick to remove started green box
            r2 = (r[0] - 2, r[1] - 2, r[2] + 2, r[3] + 2)
            self.redraw_canvas(r2)

    def delete_device(self, widget=None, device=None):
        print "Deleting %s" % device

    def buildpopupmenu(self, event, device):
        menu = gtk.Menu()
        if isinstance(device, (GuiRouter)) and device.image is None:
            menu_items = gtk.MenuItem("(no IOS)")
            menu.append(menu_items)
        elif not device.started:
            menu_items = gtk.MenuItem("Start")
            menu_items.connect("activate", self.start_device, device)
            menu.append(menu_items)
            menu_items = gtk.MenuItem("Delete")
            menu_items.connect("activate", self.delete_device, device)
            menu.append(menu_items)
        else:
            menu_items = gtk.MenuItem("Stop")
            menu_items.connect("activate", self.stop_device, device)
            menu.append(menu_items)
            if not device.suspended:
                menu_items = gtk.MenuItem("Suspend")
                menu_items.connect("activate", device.suspend, device)
                menu.append(menu_items)
            else:
                menu_items = gtk.MenuItem("Resume")
                menu_items.connect("activate", device.resume, device)
                menu.append(menu_items)
        menu_items = gtk.MenuItem("Properties")
        menu_items.connect("activate", self.show_device_properties, device)
        menu.append(menu_items)
        if isinstance(device, GuiRouter):
            if device.has_config():
                menu_items = gtk.MenuItem("Show Config")
                menu_items.connect("activate", self.show_config, device)
                menu.append(menu_items)
        menu.show_all()
        menu.popup(None, None, None, event.button, event.time)

    def buildlinkpopupmenu(self, event):
        """Display a popup menu for the currently selected links"""
        menu = gtk.Menu()
        menu_items = gtk.MenuItem("Delete")
        menu_items.connect("activate", self.delete_selected_link)
        menu.append(menu_items)
        menu.show_all()
        menu.popup(None, None, None, event.button, event.time)

    def delete_selected_link(self, event):
        """Delete the currently selected links"""
        for cct in lab.selected_ccts:
            cct.delete()
            lab.connections.remove(cct)
        self.redraw_canvas()

    #
    # menu handler
    #
    def start_all(self, widget, event=None):
        """
        Start all the devices
        """
        for d in lab.devices:
            if isinstance(d, GuiRouter):
                self.start_device(device=d)
                time.sleep(0.5) # don't overload the server :)

    def stop_all(self, widget, event=None):
        """
        Stop all the devices
        """
        for d in lab.devices:
            if isinstance(d, GuiRouter):
                self.stop_device(device=d)

    def suspend_all(self, widget, event=None):
        """
        Suspend all the devices
        """
        for d in lab.devices:
            if isinstance(d, GuiRouter) and d.started:
                d.suspend()

    def resume_all(self, widget, event=None):
        """
        Resume all the devices
        """
        for d in lab.devices:
            if isinstance(d, GuiRouter) and d.suspended:
                d.resume()


    def show_config(self, widget, event=None):
        """
        Display in a new window the router configuration that is saved
        in the net file.
        """
        if isinstance(event, GuiRouter):
            window = gtk.Window(gtk.WINDOW_TOPLEVEL)
            window.set_resizable(True)
            window.set_default_size(400, 400)
            #window.connect("destroy", lambda *w: gtk.main_quit())
            window.set_title(event.name+" configuration")
            sw = gtk.ScrolledWindow()
            sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

            textview = gtk.TextView()
            textview.set_editable(False)
            textbuffer = textview.get_buffer()
            textbuffer.set_text(event.saved_config)

            sw.add(textview)
            window.add(sw)
            window.show_all()

    def redraw_canvas(self, area=None):
        if self.window:
            alloc = self.get_allocation()
            if area is not None:
                rect = gdk.Rectangle(area[0], area[1],
                                area[2]-area[0], area[3]-area[1])
            else:
                rect = gdk.Rectangle(0, 0, alloc.width, alloc.height)
            self.window.invalidate_rect(rect, True)
            self.window.process_updates(True)

    def draw(self, context=None):
        """ Draw the canvas """
        if context is not None:
            rect = self.get_allocation()
            context.set_source_rgb(1,1,1)
            context.rectangle(rect.x,  rect.y, rect.width, rect.height)
            context.fill()
            for d in lab.devices:
                d.draw(context)
            for c in lab.connections:
                c.draw(context)

    def area_expose_cb(self, widget, event):
        self.context = widget.window.cairo_create()
        # set a clip region
        self.context.rectangle(event.area.x, event.area.y,
                               event.area.width, event.area.height)
        self.context.clip()
        self.draw(context=self.context)
        return False


