#!/usr/bin/env python

import base64
import zlib
from dynamips_lib import *
from gui_router import GuiRouter

class GuiLeopard_2FE(Leopard_2FE):
    def __init__(self, router, slot):
        Leopard_2FE.__init__(self, router, slot)
        self.technology = 'f'

    def draw(self, context, x, y):
        context.set_line_width(1)
        context.rectangle(x+2.5, y+2.5, 8, 12)
        context.move_to(x+2.5, y+8.5)
        context.line_to(x+10.5, y+8.5)
        context.stroke()
        context.move_to(x + 16, y + 13)
        context.show_text(self.adapter)

    def inside_port(self, x, y):
        return x>=2 and x <=10 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return (y>8) and 0 or 1
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        return [(6.5, 11.5), (6.5, 5.5)][port]


class Gui1PortC3600PA(object):
    def __init__(self, slot):
        self.__odd = (slot%2==1)

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.rectangle(x+2.5,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 28, y + 13)
        else:
            context.rectangle(x+42,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 8, y + 13)
        context.show_text(self.adapter[3:6])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=10 and y>=4 and y<=12
        else:
            return x>=42 and x <=50 and y>=4 and y<=12

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return 0
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return (6, 8)
        else:
            return (46, 8)

class Gui4PortC3600PA(object):
    def __init__(self, slot):
        self.__odd = (slot%2==1)

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+3.5,  y+8.5)
            context.line_to(x+13.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+38,  y+8.5)
            context.line_to(x+49.5, y+8.5)
            context.move_to(x+44,  y+2.5)
            context.line_to(x+44,  y+13.5)
            context.rectangle(x + 38, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=14 and y>=2 and y<=14
        else:
            return x>=38 and x <=50 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=(44, 8)[self.__odd]:
                if y<=8:
                    return 2
                else:
                    return 0
            else:
                if y<=8:
                    return 3
                else:
                    return 1
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return [(5.5,12.5), (11.5, 12.5), (5.5, 6.5), (11.5, 6.5)][port]
        else:
            return [(41.5,12.5), (47.5, 12.5), (41, 6), (47.5, 6.5)][port]


class GuiNM_1FE_TX(NM_1FE_TX, Gui1PortC3600PA):
    def __init__(self, router, slot):
        Gui1PortC3600PA.__init__(self, slot)
        NM_1FE_TX.__init__(self, router, slot)
        if router.chassis == '3660':
            self._Gui1PortC3600PA__odd = not self._Gui1PortC3600PA__odd
        self.technology = 'f'


class GuiNM_1E(NM_1E, Gui1PortC3600PA):
    def __init__(self, router, slot):
        Gui1PortC3600PA.__init__(self, slot)
        NM_1E.__init__(self, router, slot)
        if router.chassis == '3660':
            self._Gui1PortC3600PA__odd = not self._Gui1PortC3600PA__odd
        self.technology = 'e'


class GuiNM_4E(NM_4E, Gui4PortC3600PA):
    def __init__(self, router, slot):
        Gui4PortC3600PA.__init__(self, slot)
        NM_4E.__init__(self, router, slot)
        if router.chassis == '3660':
            self._Gui4PortC3600PA__odd = not self._Gui4PortC3600PA__odd
        self.technology = 'e'

class GuiNM_4T(NM_4T, Gui4PortC3600PA):
    def __init__(self, router, slot):
        Gui4PortC3600PA.__init__(self, slot)
        NM_4T.__init__(self, router, slot)
        if router.chassis == '3660':
            self._Gui4PortC3600PA__odd = not self._Gui4PortC3600PA__odd
        self.technology = 's'

class GuiNM_16ESW(NM_16ESW):
    def __init__(self, router, slot):
        NM_16ESW.__init__(self, router, slot)
        if router.chassis == '3660':
            self.__odd = (self.slot%2==0)
        else:
            self.__odd = (self.slot%2==1)
        self.technology = 'f'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+2.5,  y+8.5)
            context.line_to(x+50.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.move_to(x+14.5, y+2.5)
            context.line_to(x+14.5, y+13.5)
            context.move_to(x+20.5, y+2.5)
            context.line_to(x+20.5, y+13.5)
            context.move_to(x+26.5, y+2.5)
            context.line_to(x+26.5, y+13.5)
            context.move_to(x+32.5, y+2.5)
            context.line_to(x+32.5, y+13.5)
            context.move_to(x+38.5, y+2.5)
            context.line_to(x+38.5, y+13.5)
            context.move_to(x+44.5, y+2.5)
            context.line_to(x+44.5, y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 48, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+2, y+8.5)
            context.line_to(x+50.5, y+8.5)
            context.move_to(x+8, y+2)
            context.line_to(x+8, y+15)
            context.move_to(x+14, y+2)
            context.line_to(x+14, y+15)
            context.move_to(x+20, y+2)
            context.line_to(x+20, y+15)
            context.move_to(x+26, y+2)
            context.line_to(x+26, y+15)
            context.move_to(x+32, y+2)
            context.line_to(x+32, y+15)
            context.move_to(x+38, y+2)
            context.line_to(x+38, y+15)
            context.move_to(x+44, y+2)
            context.line_to(x+44, y+15)
            context.stroke()
            context.rectangle(x + 2, y + 2.5, 48, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        #context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        return x>=2 and x<=48 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return (x-2)/6 + (y<8 and 8 or 0)

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if port > 7:
            return ((port - 8) * 6 + 5, 5)
        else:
            return (port * 6 + 5, 12)




class GuiC3600(C3600, GuiRouter):

    def __init__(self, dynamips, x=10, y=10, chassis='3620', name=None):
        GuiRouter.__init__(self, x, y)
        C3600.__init__(self, dynamips, chassis, name)
        self._Router__sparsemem = False # fix dynamips
        self.colorbg_r, self.colorbg_g, self.colorbg_b = 0.5, 0.7, 0.4
        self.w = 105
        if chassis == '3620':
            self.h = 17
        elif chassis == '3640':
            self.h = 33
        elif chassis == '3660':
            self.h = 65
            # by default a 3660 has a C72000_IO_FE in slot0
            self.slot[0] = GuiLeopard_2FE(self, 0)


    def __str__(self):
        return 'C3600: '+self.name

    def draw(self, context):
        """Draw the router and all its components"""
        if self.suspended: # draw a blue halo
            context.set_source_rgb(0.2, 0.2, 0.8)
            context.rectangle(self.x, self.y, self.w, self.h)
            context.set_line_width(4.0)
            context.stroke()
        elif self.started:
            context.set_source_rgb(0.1, 0.8, 0.1)
            context.rectangle(self.x, self.y, self.w, self.h)
            context.set_line_width(4.0)
            context.stroke()

        context.set_source_rgb(self.colorbg_r, self.colorbg_g, self.colorbg_b)
        context.rectangle(self.x, self.y, self.w, self.h)
        context.fill()
        context.rectangle(self.x + 0.5, self.y + 0.5, self.w - 1.0, self.h - 1.0)
        # draw port adapter 0 / IO controller
        context.select_font_face("Sans")
        context.set_font_size(12)

        if self.h == 17: #faster check on 3620 ?
            context.move_to(self.x + self.w / 2.0, self.y)
            context.line_to(self.x + self.w / 2.0, self.y + self.h)
        elif self.h == 33:
            context.move_to(self.x + self.w / 2.0, self.y)
            context.line_to(self.x + self.w / 2.0, self.y + self.h)
            context.move_to(self.x, self.y + self.h / 2.0)
            context.line_to(self.x + self.w, self.y + self.h / 2.0)
        elif self.h == 65:
            context.move_to(self.x + self.w / 2.0, self.y)
            context.line_to(self.x + self.w / 2.0, self.y + 3 * self.h / 4 + 0.5)
            context.move_to(self.x, self.y + self.h / 4 +0.5)
            context.line_to(self.x + self.w, self.y + self.h / 4 +0.5)
            context.move_to(self.x, self.y + self.h / 2.0)
            context.line_to(self.x + self.w, self.y + self.h / 2.0)
            context.move_to(self.x, self.y + 3 * self.h / 4 + 0.5)
            context.line_to(self.x + self.w, self.y + 3 * self.h / 4 + 0.5)

        context.set_source_rgb(0,0,0)
        context.set_line_width(1.0)
        context.stroke()

        index = 0
        a = 1
        ydec = self.h - 1
        if self.chassis=='3660':
            self.slot[0].draw(context, self.x, self.y + 48)
            index = 1
            a = 0
            ydec -= 16

        xdec = self.w / 2.0
        # draw other port adapters
        for i in range(index, len(self.slot)):
            if self.slot[i]==None: # if no pa in the slot
                context.move_to(self.x + (i%2==1 and 23 or 76),
                                self.y + ydec - 2)
                context.show_text(str(i))
            else:
                # let the PA draw itself
                self.slot[i].draw(context, self.x + xdec, self.y + ydec - 16)
            if i%2==a:
                ydec -= 16
                xdec = self.w / 2.0
            else:
                xdec = 0

        context.move_to(self.x + self.w/2 - self.name_delta,
                        self.y + self.h + 12)
        context.show_text(self.name)

    def inside_pa_port(self, x, y):
        """ returns true if x, y is inside one of the device's PA's port """
        if not self.inside(x, y):
            return False
        if self.chassis == '3620':
            a = 0
        elif self.chassis == '3640':
            a = 2
            b = 0
        elif self.chassis == '3660':
            a = 5
            b = 3
        if x<self.x+self.w/2: # leftmost PA
            if y<self.y+17: # 6, 3, 1
                if (self.slot[a+1] is not None):
                    return self.slot[a+1].inside_port(x - self.x, y - self.y)
            elif y<self.y+33: # 4, 1
                if (self.slot[b+1] is not None):
                    return self.slot[b+1].inside_port(x - self.x, y - self.y - 16)
            elif y<self.y+48: # 2
                if (self.slot[1] is not None):
                    return self.slot[1].inside_port(x - self.x, y - self.y - 32)
            elif y<self.y+self.h: # 0
                if (self.slot[0] is not None):
                    return self.slot[0].inside_port(x - self.x, y - self.y - 48)
        elif x<self.x+self.w: # rightmost PA
            if y<self.y+17: # 5, 2, 0
                if self.slot[a] is not None:
                    return self.slot[a].inside_port(x - self.x - self.w/2, y - self.y)
            elif y<self.y+33: # 3, 0
                if self.slot[b] is not None:
                    return self.slot[b].inside_port(x - self.x - self.w/2,
                                             y - self.y - 16)
            elif y<self.y+48: # 1
                if self.slot[2] is not None:
                    return self.slot[2].inside_port(x - self.x - self.w/2,
                                             y - self.y - 32)
            elif y<self.y+self.h: # 0
                if (self.slot[0] is not None):
                    return self.slot[0].inside_port(x - self.x, y - self.y - 48)
        return False

    def which_pa(self, x, y):
        """ returns the slot number at position (x, y),
        and (x, y) mapped to the pa's local coordinate system
        """
        if not self.inside(x, y):
            return -1, None, None
        if self.chassis == '3620':
            a = 0
        elif self.chassis == '3640':
            a = 2
            b = 0
        elif self.chassis == '3660':
            a = 5
            b = 3
        if x<self.x+self.w/2: # leftmost PA
            if y<self.y+17 and (self.slot[a+1] is not None): # 6, 3, 1
                return a+1, x-self.x, y-self.y
            elif y<self.y+33 and (self.slot[b+1] is not None): # 4, 1
                return b+1, x-self.x, y-self.y - 16
            elif y<self.y+48 and (self.slot[2] is not None): # 2
                return 2, x-self.x, y-self.y - 32
            elif y<self.y+self.h and (self.slot[0] is not None): # 0
                return 0, x-self.x, y-self.y - 48
        else: # rightmost PA
            if y<self.y+17 and self.slot[a] is not None: #5, 2, 0
                return a, x-self.x-self.w/2, y-self.y
            elif y<self.y+33 and self.slot[b] is not None: # 3, 0
                return b, x-self.x-self.w/2, y-self.y-16
            elif y<self.y+48 and self.slot[1] is not None: # 1
                return 1, x-self.x-self.w/2, y-self.y-32
            elif y<self.y+self.h and (self.slot[0] is not None): # 0
                return 0, x-self.x, y-self.y - 48
        return -1, None, None


    def getPortPos(self, pa, port):
        """ Returns the center position (x, y) of the given pa/port """
        (x, y) = self.slot[pa].getPortPos(port)
        if self.chassis == '3660':
            if pa in [1, 2]:
                y += 32
            elif pa in [3, 4]:
                y += 16
            if pa%2==0:
                x += self.w / 2
        elif self.chassis == '3640':
            if pa in [0, 1]:
                y += 16
            if pa%2==0:
                x += self.w / 2
        else:
            if pa%2==1:
                x += self.w / 2
        return x+self.x, y+self.y



    def dirty_area(self):
        """ Returns a (x0, y0, x1, y1) dirty area """
        minx = miny = 60000
        maxx = maxy = 0
        for l,l,c,s in self.connections:
            minx = min(minx, c.xa, c.xb)
            maxx = max(maxx, c.xa, c.xb)
            miny = min(miny, c.ya, c.yb)
            maxy = max(maxy, c.ya, c.yb)
        minx = min(minx, self.x, self.px)
        maxx = max(maxx, self.x + self.w, self.px + self.w)
        miny = min(miny, self.y, self.py)
        maxy = max(maxy, self.y + self.h, self.py + self.h)
        if self.started:
            return (minx - 2, miny - 2, maxx + 2, maxy + 18)
        return (minx, miny, maxx, maxy+16) # for the router's name


    def gui_start(self):
        if not self.started:
            C3600.start(self)
            if self.push_cfg:
                self.config_gz64 = self.tmp_config64bz
                self.tmp_config64bz = None
                C3600.stop(self)
                C3600.start(self)
                self.push_cfg = False
            self.stopped_config64bz = None
            self.started = True
            return True


    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def gui_stop(self):
        if self.started:
            #FIXME: dynamips bug, retrieve the config before stopping the device
            try:
                config_b64 = self.config_b64
                config = base64.b64decode(config_b64)
                config_gz = zlib.compress(config)
                self.stopped_config64bz = base64.b64encode(config_gz)
            except:
                pass
            # stop the device
            C3600.stop(self)
            self.started = False
            return True

    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def suspend(self, widget=None, event=None):
        if self.started and not self.suspended:
            C3600.suspend(self)
            self.suspended = True

    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def resume(self, widget=None, event=None):
        if self.suspended:
            C3600.resume(self)
            self.suspended = False


