#!/usr/bin/env python

"""
gui_router.py
Copyright (C) 2006  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import base64
import cairo
import os
import sys
import zlib
from dynamips_lib import *
from gui_router import GuiRouter, dynamips

class GuiPA_FE_TX(PA_FE_TX):
    def __init__(self, router, slot):
        PA_FE_TX.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 'f'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.rectangle(x+2.5,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 28, y + 13)
        else:
            context.rectangle(x+53.5,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 8, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=10 and y>=4 and y<=12
        else:
            return x>=53 and x <=61 and y>=4 and y<=12

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return 0
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return (6,8)
        else:
            return (57,8)


class GuiPA_4E(PA_4E):
    def __init__(self, router, slot):
        PA_4E.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 'e'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+3.5,  y+8.5)
            context.line_to(x+13.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+50.5,  y+8.5)
            context.line_to(x+60.5, y+8.5)
            context.move_to(x+55.5,  y+2.5)
            context.line_to(x+55.5,  y+13.5)
            context.rectangle(x + 49.5, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=14 and y>=2 and y<=14
        else:
            return x>=49 and x <=61 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=(55,8)[self.__odd]:
                if y<=8:
                    return 2
                else:
                    return 0
            else:
                if y<=8:
                    return 3
                else:
                    return 1
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return [(5,11), (11, 11), (5, 5), (11, 5)][port]
        else:
            return [(52,11), (58, 11), (52, 5), (58, 5)][port]


class GuiPA_8E(PA_8E):
    def __init__(self, router, slot):
        PA_8E.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 'e'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+3.5,  y+8.5)
            context.line_to(x+25.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.move_to(x+14.5, y+2.5)
            context.line_to(x+14.5, y+13.5)
            context.move_to(x+20.5, y+2.5)
            context.line_to(x+20.5, y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 24, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+37.5, y+8.5)
            context.line_to(x+61.5, y+8.5)
            context.move_to(x+43.5, y+2.5)
            context.line_to(x+43.5, y+13.5)
            context.move_to(x+49.5, y+2.5)
            context.line_to(x+49.5, y+13.5)
            context.move_to(x+55.5, y+2.5)
            context.line_to(x+55.5, y+13.5)
            context.stroke()
            context.rectangle(x + 37.5, y + 2.5, 24, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x<=24 and y>=2 and y<=14
        else:
            return x>=37 and x<=61 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=(43,8)[self.__odd]:
                if y<=8:
                    return 4
                else:
                    return 0
            elif x<=(49,14)[self.__odd]:
                if y<=8:
                    return 5
                else:
                    return 1
            elif x<=(55,20)[self.__odd]:
                if y<=8:
                    return 6
                else:
                    return 2
            else:
                if y<=8:
                    return 7
                else:
                    return 3

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return [(5, 11), (11, 11), (17, 11), (23, 11), (5, 5), (11, 5), (17, 5), (23, 5)][port]
        else:
            return [(40,11), (46, 11), (52, 11), (58, 11), (40, 5), (46, 5), (52, 5), (58, 5)][port]


class GuiPA_A1(PA_A1):
    def __init__(self, router, slot):
        PA_A1.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 'a'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.rectangle(x+2.5,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.rectangle(x+53.5,  y+4.5, 8, 8)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=10 and y>=4 and y<=12
        else:
            return x>=53 and x <=61 and y>=4 and y<=12

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return 0
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return (6,8)
        else:
            return (57,8)


class GuiPA_4Tp(PA_4T):
    def __init__(self, router, slot):
        PA_4T.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 's'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+3.5,  y+8.5)
            context.line_to(x+13.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+50.5,  y+8.5)
            context.line_to(x+60.5, y+8.5)
            context.move_to(x+55.5,  y+2.5)
            context.line_to(x+55.5,  y+13.5)
            context.rectangle(x + 49.5, y + 2.5, 12, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x <=14 and y>=2 and y<=14
        else:
            return x>=49 and x <=61 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=(55,8)[self.__odd]:
                if y<=8:
                    return 2
                else:
                    return 0
            else:
                if y<=8:
                    return 3
                else:
                    return 1
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return [(5,11), (11, 11), (5, 5), (11, 5)][port]
        else:
            return [(52,11), (58, 11), (52, 5), (58, 5)][port]


class GuiPA_8T(PA_8T):
    def __init__(self, router, slot):
        PA_8T.__init__(self, router, slot)
        self.__odd = (self.slot%2==1)
        self.technology = 's'

    def draw(self, context, x, y):
        context.set_line_width(1)
        if self.__odd:
            context.move_to(x+3.5,  y+8.5)
            context.line_to(x+25.5, y+8.5)
            context.move_to(x+8.5,  y+2.5)
            context.line_to(x+8.5,  y+13.5)
            context.move_to(x+14.5, y+2.5)
            context.line_to(x+14.5, y+13.5)
            context.move_to(x+20.5, y+2.5)
            context.line_to(x+20.5, y+13.5)
            context.rectangle(x + 2.5, y + 2.5, 24, 12)
            context.stroke()
            context.move_to(x + 30, y + 13)
        else:
            context.move_to(x+37.5, y+8.5)
            context.line_to(x+61.5, y+8.5)
            context.move_to(x+43.5, y+2.5)
            context.line_to(x+43.5, y+13.5)
            context.move_to(x+49.5, y+2.5)
            context.line_to(x+49.5, y+13.5)
            context.move_to(x+55.5, y+2.5)
            context.line_to(x+55.5, y+13.5)
            context.stroke()
            context.rectangle(x + 37.5, y + 2.5, 24, 12)
            context.stroke()
            context.move_to(x + 12, y + 13)
        context.show_text(self.adapter[3:])

    def inside_port(self, x, y):
        if self.__odd:
            return x>=2 and x<=24 and y>=2 and y<=14
        else:
            return x>=37 and x<=61 and y>=2 and y<=14

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=(43,8)[self.__odd]:
                if y<=8:
                    return 4
                else:
                    return 0
            elif x<=(49,14)[self.__odd]:
                if y<=8:
                    return 5
                else:
                    return 1
            elif x<=(55,20)[self.__odd]:
                if y<=8:
                    return 6
                else:
                    return 2
            else:
                if y<=8:
                    return 7
                else:
                    return 3

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if self.__odd:
            return [(5, 11), (11, 11), (17, 11), (23, 11), (5, 5), (11, 5), (17, 5), (23, 5)][port]
        else:
            return [(40,11), (46, 11), (52, 11), (58, 11), (40, 5), (46, 5), (52, 5), (58, 5)][port]



class GuiC7200_IO_FE(PA_C7200_IO_FE):
    def __init__(self, router, slot):
        PA_C7200_IO_FE.__init__(self, router, slot)
        self.technology = 'f'

    def draw(self, context, x, y):
        context.set_line_width(1)
        context.rectangle(x + 2.5, y + 4.5, 8, 8)
        context.stroke()
        context.move_to(x + 27, y + 13)
        context.show_text(self.adapter)

    def inside_port(self, x, y):
        return x>=2 and x <=10 and y>=4 and y<=12

    def which_port(self, x, y):
        if self.inside_port(x, y):
            return 0
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        return (6, 8)

class GuiC7200_IO_2FE(PA_C7200_IO_2FE):
    def __init__(self, router, slot):
        PA_C7200_IO_2FE.__init__(self, router, slot)
        self.technology = 'f'

    def draw(self, context, x, y):
        context.set_line_width(1)
        context.rectangle(x + 2.5, y + 4.5, 8, 8)
        context.rectangle(x + 10.5, y + 4.5, 8, 8)
        context.stroke()
        context.move_to(x + 27, y + 13)
        context.show_text(self.adapter)

    def inside_port(self, x, y):
        return x>=2 and x <=18 and y>=4 and y<=12

    def which_port(self, x, y):
        if self.inside_port(x, y):
            if x<=10:
                return 0
            else:
                return 1
        return -1

    def getPortPos(self, port):
        """ Returns the local, center coordinates (x, y) of the given port """
        if port==0:
            return (6, 8)
        else:
            return (14,8)


class GuiC7200(C7200, GuiRouter):
    pixmap = cairo.ImageSurface.create_from_png(
                    os.path.join('/usr/local/share/dynagui/img', 'chassis.png'))

    def __init__(self, dynamips, x=10, y=10, name=None):
        GuiRouter.__init__(self, x, y)
        C7200.__init__(self, dynamips=dynamips, name=name)
        self._Router__sparsemem = False # fix dynamips

        # width and height of image
        self.w, self.h = GuiC7200.pixmap.get_width(), \
                         GuiC7200.pixmap.get_height()
        self.colorbg_r, self.colorbg_g, self.colorbg_b = 0.4, 0.6, 0.6

        # by default a 7200 has a C72000_IO_FE in slot0
        #self.slot[0] = GuiC7200_IO_FE(self, 0)

    def __str__(self):
        return 'C7200: '+self.name

    def gui_start(self):
        if not self.started:
            C7200.start(self)
            if self.push_cfg:
                self.config_gz64 = self.tmp_config64bz
                self.tmp_config64bz = None
                C7200.stop(self)
                C7200.start(self)
                self.push_cfg = False
            self.stopped_config64bz = None
            self.started = True
            return True


    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def gui_stop(self):
        if self.started:
            #FIXME: dynamips bug, retrieve the config before stopping the device
            try:
                config_b64 = self.config_b64
                config = base64.b64decode(config_b64)
                config_gz = zlib.compress(config)
                self.stopped_config64bz = base64.b64encode(config_gz)
            except:
                print 'Error extracting config: '
                print sys.exc_info()
                pass
            # stop the device
            C7200.stop(self)
            self.started = False
            return True

    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def suspend(self, widget=None, event=None):
        if self.started and not self.suspended:
            C7200.suspend(self)
            self.suspended = True

    # FIXME this is a GUI callback.. needs to be moved to dynagui
    def resume(self, widget=None, event=None):
        if self.suspended:
            C7200.resume(self)
            self.suspended = False

    def which_pa(self, x, y):
        """ returns the slot number at position (x, y),
        and (x, y) mapped to the pa's local coordinate system
        """
        if not self.inside(x, y):
            return -1, None, None
        if x>self.x and y>self.y:
            if x<self.x+63: # leftmost PA
                if y<self.y+16 and (self.slot[5] is not None):
                    return 5, x-self.x, y-self.y
                elif y<self.y+32 and (self.slot[3] is not None):
                    return 3, x-self.x, y-self.y - 16
                elif y<self.y+48 and (self.slot[1] is not None):
                    return 1, x-self.x, y-self.y - 32
                elif y<self.y+self.h and (self.slot[0] is not None):
                    return 0, x-self.x, y-self.y - 48
            elif x<self.x+self.w: # rightmost PA
                if y<self.y+16 and self.slot[6] is not None:
                    return 6, x-self.x-63, y-self.y
                elif y<self.y+32 and self.slot[4] is not None:
                    return 4, x-self.x-63, y-self.y-16
                elif y<self.y+48 and self.slot[2] is not None:
                    return 2, x-self.x-63, y-self.y-32
                elif y<self.y+self.h and (self.slot[0] is not None):
                    return 0, x-self.x, y-self.y - 48
        return -1, None, None


    def inside_pa_port(self, x, y):
        """ returns true if x, y is inside one of the device's PA's port """
        if not self.inside(x, y):
            return False
        if x>self.x and y>self.y:
            if x<self.x+63: # leftmost PA
                if y<self.y+16 and (self.slot[5] is not None):
                    return self.slot[5].inside_port(x - self.x, y - self.y)
                elif y<self.y+32 and (self.slot[3] is not None):
                    return self.slot[3].inside_port(x - self.x, y - self.y - 16)
                elif y<self.y+48 and (self.slot[1] is not None):
                    return self.slot[1].inside_port(x - self.x, y - self.y - 32)
                elif y<self.y+self.h and (self.slot[0] is not None):
                    return self.slot[0].inside_port(x - self.x, y - self.y - 48)
            elif x<self.x+self.w: # rightmost PA
                if y<self.y+16 and self.slot[6] is not None:
                    return self.slot[6].inside_port(x - self.x - 63, y - self.y)
                elif y<self.y+32 and self.slot[4] is not None:
                    return self.slot[4].inside_port(x - self.x - 63,
                                                 y - self.y - 16)
                elif y<self.y+48 and self.slot[2] is not None:
                    return self.slot[2].inside_port(x - self.x - 63,
                                                 y - self.y - 32)
                elif y<self.y+self.h and (self.slot[0] is not None):
                    return self.slot[0].inside_port(x - self.x, y - self.y - 48)
        return False

    def inside_empty_port(self, x, y):
        """ returns true if x, y is inside one of the device's empty PA's port """
        slot_nb, xl, yl = self.which_pa(x, y)
        slot_pa = self.slot[slot_nb]
        if slot_pa is not None:
            port = slot_pa.which_port(xl, yl)
            for lpa,lpo,c,s in self.connections:
                if slot_nb==lpa and port==lpo:
                    return False
        return True

    def getPortPos(self, pa, port):
        """ Returns the center position (x, y) of the given pa/port """
        (x, y) = self.slot[pa].getPortPos(port)
        if pa==0:
            y += 48
        elif pa in [1, 2]:
            y += 32
        elif pa in [3, 4]:
            y += 16
        if pa==0:
            pass
        elif pa%2==0 and pa != 0:
            x += 64
        return x+self.x, y+self.y

    def draw(self, context):
        """Draw the router and all its components"""
        if self.suspended: # draw a blue halo
            context.set_source_rgb(0.2, 0.2, 0.8)
            context.rectangle(self.x, self.y, self.w, self.h)
            context.set_line_width(4.0)
            context.stroke()
        elif self.started: #draw a green halo
            context.set_source_rgb(0.1, 0.8, 0.1)
            context.rectangle(self.x, self.y, self.w, self.h)
            context.set_line_width(4.0)
            context.stroke()
        context.set_source_rgb(self.colorbg_r, self.colorbg_g, self.colorbg_b)
        context.rectangle(self.x, self.y, self.w, self.h)
        context.fill()
        context.set_source_surface(self.pixmap, self.x, self.y)
        context.paint()
        context.set_source_rgb(0,0,0)
        # draw port adapter 0 / IO controller
        context.select_font_face("Sans")
        context.set_font_size(12)

        if self.slot[0]==None:
            context.move_to(self.x + 60, self.y + 62)
            context.show_text('0')
        else:
            # let the PA draw itself
            self.slot[0].draw(context, self.x, self.y + 48)

        # draw other port adapters
        ydec = 48
        for i in range(1, 7):
            if self.slot[i]==None: # if no pa in the slot
                context.move_to(self.x + (i%2==1 and 30 or 90),
                                self.y + ydec - 2)
                context.show_text(str(i))
            else:
                # let the PA draw itself
                if i%2==0:
                    xdec = 63
                else:
                    xdec = 0
                self.slot[i].draw(context, self.x + xdec, self.y + ydec - 16)
            if i%2==0:
                ydec -= 16

        context.move_to(self.x + self.w/2 - self.name_delta,
                        self.y + self.h + 12)
        context.show_text(self.name)

    def dirty_area(self):
        """ Returns a (x0, y0, x1, y1) dirty area """
        minx = miny = 60000
        maxx = maxy = 0
        for l,l,c,s in self.connections:
            minx = min(minx, c.xa, c.xb)
            maxx = max(maxx, c.xa, c.xb)
            miny = min(miny, c.ya, c.yb)
            maxy = max(maxy, c.ya, c.yb)
        minx = min(minx, self.x, self.px)
        maxx = max(maxx, self.x + self.w, self.px + self.w)
        miny = min(miny, self.y, self.py)
        maxy = max(maxy, self.y + self.h, self.py + self.h)
        if self.started:
            return (minx - 2, miny - 2, maxx + 2, maxy + 18)
        return (minx, miny, maxx, maxy+16) # for the router's name

    def intersect(self, area):
        left, top, right, bottom = area
        l, t, r, b = self.x, self.y, self.x+self.w, self.y+self.h
        if left < l: left = l
        if top < t: top = t
        if right > r: right = r
        if bottom > b: bottom = b
        return not (left >= right or top >= bottom)


