#!/usr/bin/env python
"""
nvram_io.py
Copyright (C) 2006  Yannick Le Teigner
Based on nvram_export.c (dynamips) from Christophe Fillot.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import sys
import struct
import zlib
import base64


def extract_conf(filename):
    C7200_NVRAM_ROM_RES_SIZE = 2048
    C7200_NVRAM_ADDR = 0x1e000000

    file = open(filename, 'rb')

    file.seek(C7200_NVRAM_ROM_RES_SIZE+6)
    tag = struct.unpack("!I", file.read(4))[0]
    if not tag==0xF0A5ABCD:
        print "Cannot find IOS tag"
        sys.exit(0)

    file.seek(0x06, 1)
    start = struct.unpack("!I", file.read(4))[0] + 1
    end = struct.unpack("!I", file.read(4))[0]
    nvlen = struct.unpack("!I", file.read(4))[0]

    if (start<C7200_NVRAM_ADDR or end<C7200_NVRAM_ADDR or end<start):
        print "Error: invalid configuration size"
        print sys.exit(0)

    len = end - start
    if len+1 != nvlen:
        print "NVRAM: invalid configuration size"
        print sys.exit(0)

    start -= C7200_NVRAM_ADDR
    file.seek(start)
    conf = file.read(len)
    file.close()
    return conf


def extract_conf_b64gz(filename):
    """Read the config from the nvram file passed as an argument
    and returns a gzipped base64 encoded configuration string"""
    conf = extract_conf(filename)
    confgz = zlib.compress(conf)
    return base64.b64encode(confgz)


def read_conf_b64gz(confgz64):
    """Returns a decompressed (zlib) and decoded (b64) string given
    as an argument"""
    confgz = base64.b64decode(confgz64)
    return zlib.decompress(confgz)


if __name__=="__main__":
    print extract_conf_b64gz(sys.argv[1])
